﻿using System;
using System.Net.Http;
using System.Text;
using Newtonsoft.Json;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace KingWebservicesClientDemo
{
    [TestClass]
    public class KingWebServicesClientTests
    {
        private static class Constanten
        {
            public static readonly string MediaType = "application/json";
            public static readonly Encoding Encoding = Encoding.UTF8;
            public static readonly string ToegangscodeHttpHeader = "ACCESS-TOKEN";
        }

        private static void executeWebservice(HttpClient httpClient, string host, string administratiecode, string webservicenaam, dynamic data)
        {
            HttpContent jsonContent = new StringContent(JsonConvert.SerializeObject(data), Constanten.Encoding, Constanten.MediaType);
            HttpResponseMessage response = httpClient.PostAsync($"{host}/{administratiecode}/{webservicenaam}", jsonContent).Result;
            object content = JsonConvert.DeserializeObject(response.Content.ReadAsStringAsync().Result);
            Console.WriteLine($"Resultaat {webservicenaam}: {content}");
        }

        [TestMethod]
        public void test_proces_aanmaken_order_inclusief_twee_orderregels_en_een_colloregel()
        {
            bool testSuccesvol;
            string foutmelding = string.Empty;

            //=============================================================//
            // LET OP -- DEZE VARIABELEN MOETEN WORDEN AANGEPAST -- LET OP //            
            //=============================================================//
            string host = "http://localhost:8082/";          // -> PC-naam, ip of localhost + de poort gekozen is in IIS.
            string toegangscode = "MijnGeheimeToegangscode"; // -> Stel deze toegangscode (voor testdoeleinden) in King 5 in bij de vaste gegevens webservices.
            string administratiecode = "DemoArt";            // -> De administratie waarin de gegevens toegevoegd worden.
            //=============================================================//


            try
            {
                using (var httpClient = new HttpClient())
                {
                    //eenmalig de toegangscode-httpheader aan de client toevoegen.
                    httpClient.DefaultRequestHeaders.Add(Constanten.ToegangscodeHttpHeader, toegangscode);

                    long ordernummer = 20160007;
                    var order = StaticOrderData.MaakOrderObject(ordernummer);
                    var orderregel1 = StaticOrderData.MaakOrderregelObject(ordernummer, "Excel");
                    var orderregel2 = StaticOrderData.MaakOrderregelObject(ordernummer, "Word");
                    var ordercollo = StaticOrderData.MaakOrdercolloObject(ordernummer);
                    var orderlock = StaticOrderData.MaakOrderlockObject(ordernummer);

                    executeWebservice(httpClient, host, administratiecode, "Order_toevoegen", order);
                    executeWebservice(httpClient, host, administratiecode, "Order_OrderRegel_Toevoegen", orderregel1);
                    executeWebservice(httpClient, host, administratiecode, "Order_OrderRegel_Toevoegen", orderregel2);
                    executeWebservice(httpClient, host, administratiecode, "Order_Collo_Toevoegen", ordercollo);
                    executeWebservice(httpClient, host, administratiecode, "Order_LockVrijgeven", orderlock);
                }
                testSuccesvol = true;
            }
            catch (Exception ex)
            {
                testSuccesvol = false;
                foutmelding = ex.Message;
            }

            Assert.IsTrue(testSuccesvol, $"Er is een fout opgetreden: {foutmelding}");
        }
    }
}